// Пример, демонстрирующий работу 
// модуля ЦАП QMS45 в динамическом режиме 
// (генерирует синусоиды с подкачкой данных из ПК)
// ЦАП работают на частоте 125 кГц/канал.


#include "../../dll/qmx.h"
#include <shellapi.h>
#include <stdio.h>
#include <conio.h> 
#include <math.h>


HANDLE SD;                              // Дескриптор системы
QMX_CONFIG CFG;                         // Конфигурация системы
QMX_CC_F CC_Dac1[8];					// Калибровочные коэффициенты


static unsigned int out_cntr=0;
static unsigned short int *out_buf[2];

void CheckErr(void);
void quit(void);
void FillBuf(WORD *buf, DWORD len);




//void main(void)
void main(int argc, char* argv[])
{
	char s_wrk[64];
	int i_wrk;
	double CRate;

	printf("Please wait....\r");
	
	// Запустим сервер
	ShellExecuteW(NULL, TEXT("open"), TEXT("qmbox_ds.exe"), TEXT("-p3"), NULL, SW_SHOWNORMAL);
	Sleep(500);

	// Получим дискриптор системы
	SD = 0;
	SD=QMX_CircBufAttach(0);
	if (SD == NULL) {
		printf("ERROR: No QMBox found.");
		quit(); 
	}

	// Проинициализуем систему
	QMX_InitModules(SD, QMX_LOG_NONE, NULL);
	CheckErr();

	// Фиктивный вызов QMX_Prepare - выясним конфигурацию устройства
	// т.е. из каких модулей оно состоит
	QMX_MOD_SetActive(SD, 0, QMX_ON);
	QMX_Prepare(SD, 100, &CFG);
	CheckErr();
	QMX_MOD_SetActive(SD, 0, QMX_OFF);

	
	// Убедимся, что первый модуль системы - QMS45
	strncpy_s (s_wrk, CFG.ModName[0], 5); s_wrk[5] = 0;
	if (!_stricmp(s_wrk, "QMS45"))	printf("Module name OK\n");
	else{ printf("Module Name error\n");	quit();	}


	// Определим нужные указатели
	out_buf[0]=(WORD *)CFG.CBOutput.buf_start;								// указатель на первую половину буфера вывода
	out_buf[1]=(WORD *)(CFG.CBOutput.buf_start+(*CFG.CBOutput.buf_size>>1));	// указатель на вторую половину буфера вывода

	// Установим частоту работы ЦАПа
	QMX_QMS45_SetDacRate(SD, 0, 1000000.0, &CRate);
	printf("DAC Rate = %8.2f Hz \n", CRate);
	
	//Считаем калибровочные коэффициенты для ЦАП
	//для всех его 8ми каналов
	for (i_wrk = 0; i_wrk < 8; i_wrk++) QMX_QMS45_ReadCC(SD, 0, i_wrk, &CC_Dac1[i_wrk]);


	// Теперь осуществим предварительную заливку буфера динамического ЦАП
	// для того, чтобы к моменту "Старта" в его локальном буфере уже были наготове нужные данные
	QMX_MOD_SetActive(SD, 0, QMX_ON);	// Для этого он объявляется активным
	QMX_Prepare(SD, 0, &CFG);			// После этого обязательно вызвать QMX_Prepare
	CheckErr();
	FillBuf(out_buf[0], *CFG.CBOutput.buf_size);		// Заполняем весь буфер вывода (обе половины) нужными данными
	QMX_MOD_OutputPrestage(SD, 0);						// Заливаем этот буфер в модуль


//	QMX_MOD_SetStartMode(SD, 0xFF, QMX_START_MODE_EXTERNAL);

	// Запустим сеанс сбора данных - дадим "Старт"
	QMX_Start(SD);
	CheckErr();


	while(!_kbhit()){				// Бесконечный цикл (до нажатия кнопки)				
	  if(out_cntr<*CFG.CBOutput.buf_ptr) {								// Если в сервере есть свободный буфер для отправки						
		   FillBuf(out_buf[out_cntr++&1], (*CFG.CBOutput.buf_size)>>1); // Заполним его очередной порцией данных
		   QMX_CircBufOutputUpdate(SD);									// И дадим команду серверу на отправку
		   printf("Write Counter: %3u\r", out_cntr);	// Отобразим счетчик отправленных половинок буферов
	  }
	  Sleep(10);
	}

	quit();											// выход из программы
}


// Выход из программы
void quit(void){
	
	printf("\n\n Please wait....\r");
	QMX_Stop(SD, QMX_STOP_ALL);			// Остановим систему
	QMX_MOD_SetStartMode(SD, 0xFF, QMX_START_MODE_MANUAL); 									
										
	QMX_CircBufDetach(SD);              // Освободим виртуальный слот USB
										// Остановим сервер
	ShellExecute(NULL, TEXT("open"), TEXT("qmbox_ds.exe"), TEXT("-s0"), NULL, SW_SHOWNORMAL);
	printf("\n\n");
	ExitProcess(0);
}


// Проверка на ошибки
void CheckErr(void) {
char tmp_str[1024];

if (QMX_GetLastError(SD, tmp_str, sizeof(tmp_str))==QMX_OK){ return;}
else  {	printf("%s\n", tmp_str);	quit();}
}



// Заполнение очередной порции данных для динамического ЦАП
void FillBuf(WORD *buf, DWORD len) {
#define _2PI	6.283185307179586476925286766559
DWORD i;
static double phase[8]={0.7, 1.5, 2.3, 2.9, 3.6, 4.3, 5.0, 5.7};
static double amp[8]={16000., 15000., 14000., 13000., 12000., 10000., 8000., 4000.}; 
static double freq[8]={0.2, 0.05, 0.1, 0.15, 0.2, 0.25, 0.3, 0.35};
static double j[8]={0};
 
  for(i=0; i<len; i++) *buf++=(WORD)(32767.-amp[i&0x7]*sin(phase[i&0x7]+(j[i&0x7]+=freq[i&0x7])>_2PI?j[i&0x7]-=_2PI:j[i&0x7]));
}




